# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Challenge Service."""

__version__ = "1.30.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .admin_create_challenge import AdminCreateChallenge
from .admin_delete_challenge import AdminDeleteChallenge
from .admin_delete_tied_challenge import AdminDeleteTiedChallenge
from .admin_get_active_challenges import AdminGetActiveChallenges
from .admin_get_challenge import AdminGetChallenge
from .admin_get_challenges import AdminGetChallenges
from .admin_get_challenges import (
    SortByEnum as AdminGetChallengesSortByEnum,
    StatusEnum as AdminGetChallengesStatusEnum,
)
from .admin_get_periods import AdminGetPeriods
from .admin_randomize_challenge import AdminRandomizeChallenge
from .admin_update_challenge import AdminUpdateChallenge
from .admin_update_tied_chall_70da14 import AdminUpdateTiedChallengeSchedule



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# Analytics Game Telemetry

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from .....core import Operation
from .....core import HeaderStr
from .....core import HttpResponse

from ...models import BaseErrorResponse
from ...models import PlayTimeResponse


class ProtectedUpdatePlaytimeGameTelemetryV1ProtectedSteamIdsSteamIdPlaytimePlaytimePut(
    Operation
):
    """Protected Update Playtime (protected_update_playtime_game_telemetry_v1_protected_steamIds__steamId__playtime__playtime__put)

    This endpoint requires valid JWT token.
    This endpoint does not require permission.

    This endpoint update player's total playtime in a specific game (AppId) from service's cache.

    Properties:
        url: /game-telemetry/v1/protected/steamIds/{steamId}/playtime/{playtime}

        method: PUT

        tags: ["Gametelemetry Operations"]

        consumes: []

        produces: ["application/json"]

        securities: [COOKIE_AUTH] or [BEARER_AUTH]

        cookie: (Cookie) OPTIONAL Union[str, HeaderStr] in header

        playtime: (playtime) REQUIRED str in path

        steam_id: (steamId) REQUIRED str in path

    Responses:
        200: OK - PlayTimeResponse (Successful Response)

        404: Not Found - BaseErrorResponse (User not found)

        422: Unprocessable Entity - BaseErrorResponse (Unable to process request)

        500: Internal Server Error - BaseErrorResponse (Internal Server Error)
    """

    # region fields

    _url: str = "/game-telemetry/v1/protected/steamIds/{steamId}/playtime/{playtime}"
    _method: str = "PUT"
    _consumes: List[str] = []
    _produces: List[str] = ["application/json"]
    _securities: List[List[str]] = [["COOKIE_AUTH"], ["BEARER_AUTH"]]
    _location_query: str = None

    cookie: Union[str, HeaderStr]  # OPTIONAL in [header]
    playtime: str  # REQUIRED in [path]
    steam_id: str  # REQUIRED in [path]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "header": self.get_header_params(),
            "path": self.get_path_params(),
        }

    def get_header_params(self) -> dict:
        result = {}
        if hasattr(self, "cookie"):
            result["Cookie"] = self.cookie
        return result

    def get_path_params(self) -> dict:
        result = {}
        if hasattr(self, "playtime"):
            result["playtime"] = self.playtime
        if hasattr(self, "steam_id"):
            result["steamId"] = self.steam_id
        return result

    # endregion get_x_params methods

    # region is/has methods

    # endregion is/has methods

    # region with_x methods

    def with_cookie(
        self, value: Union[str, HeaderStr]
    ) -> ProtectedUpdatePlaytimeGameTelemetryV1ProtectedSteamIdsSteamIdPlaytimePlaytimePut:
        self.cookie = value
        return self

    def with_cookie_access_token(
        self, value: str
    ) -> ProtectedUpdatePlaytimeGameTelemetryV1ProtectedSteamIdsSteamIdPlaytimePlaytimePut:
        if not hasattr(self, "cookie"):
            self.cookie = HeaderStr()
        self.cookie["access_token"] = value
        return self

    def with_playtime(
        self, value: str
    ) -> ProtectedUpdatePlaytimeGameTelemetryV1ProtectedSteamIdsSteamIdPlaytimePlaytimePut:
        self.playtime = value
        return self

    def with_steam_id(
        self, value: str
    ) -> ProtectedUpdatePlaytimeGameTelemetryV1ProtectedSteamIdsSteamIdPlaytimePlaytimePut:
        self.steam_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "cookie") and self.cookie:
            result["Cookie"] = str(self.cookie)
        elif include_empty:
            result["Cookie"] = ""
        if hasattr(self, "playtime") and self.playtime:
            result["playtime"] = str(self.playtime)
        elif include_empty:
            result["playtime"] = ""
        if hasattr(self, "steam_id") and self.steam_id:
            result["steamId"] = str(self.steam_id)
        elif include_empty:
            result["steamId"] = ""
        return result

    # endregion to methods

    # region response methods

    # noinspection PyMethodMayBeStatic
    def parse_response(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[
        Union[None, PlayTimeResponse], Union[None, BaseErrorResponse, HttpResponse]
    ]:
        """Parse the given response.

        200: OK - PlayTimeResponse (Successful Response)

        404: Not Found - BaseErrorResponse (User not found)

        422: Unprocessable Entity - BaseErrorResponse (Unable to process request)

        500: Internal Server Error - BaseErrorResponse (Internal Server Error)

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 200:
            return PlayTimeResponse.create_from_dict(content), None
        if code == 404:
            return None, BaseErrorResponse.create_from_dict(content)
        if code == 422:
            return None, BaseErrorResponse.create_from_dict(content)
        if code == 500:
            return None, BaseErrorResponse.create_from_dict(content)

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(
        cls,
        playtime: str,
        steam_id: str,
        cookie: Optional[Union[str, HeaderStr]] = None,
        **kwargs,
    ) -> ProtectedUpdatePlaytimeGameTelemetryV1ProtectedSteamIdsSteamIdPlaytimePlaytimePut:
        instance = cls()
        instance.playtime = playtime
        instance.steam_id = steam_id
        if cookie is not None:
            instance.cookie = cookie
        if x_flight_id := kwargs.get("x_flight_id", None):
            instance.x_flight_id = x_flight_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ProtectedUpdatePlaytimeGameTelemetryV1ProtectedSteamIdsSteamIdPlaytimePlaytimePut:
        instance = cls()
        if "Cookie" in dict_ and dict_["Cookie"] is not None:
            instance.cookie = str(dict_["Cookie"])
        elif include_empty:
            instance.cookie = ""
        if "playtime" in dict_ and dict_["playtime"] is not None:
            instance.playtime = str(dict_["playtime"])
        elif include_empty:
            instance.playtime = ""
        if "steamId" in dict_ and dict_["steamId"] is not None:
            instance.steam_id = str(dict_["steamId"])
        elif include_empty:
            instance.steam_id = ""
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "Cookie": "cookie",
            "playtime": "playtime",
            "steamId": "steam_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "Cookie": False,
            "playtime": True,
            "steamId": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Lobby Server."""

__version__ = "3.43.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .admin_delete_global_config import AdminDeleteGlobalConfig
from .admin_get_global_config import AdminGetGlobalConfig
from .admin_update_global_config import AdminUpdateGlobalConfig
from .create_template import CreateTemplate
from .delete_template_localization import DeleteTemplateLocalization
from .delete_template_slug import DeleteTemplateSlug
from .free_form_notification import FreeFormNotification
from .get_game_template import GetGameTemplate
from .get_localization_template import GetLocalizationTemplate
from .get_slug_template import GetSlugTemplate
from .notification_with_template import NotificationWithTemplate
from .publish_template import PublishTemplate
from .update_localization_template import UpdateLocalizationTemplate



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import deprecated
from ....core import same_doc_as

from ..models import ApimodelAppItem
from ..models import ApimodelApplyAppConfigRequest
from ..models import ApimodelCreateAppV2Request
from ..models import ApimodelGetAppListV2Request
from ..models import ApimodelGetAppListV2Response
from ..models import ApimodelIncreaseLimitFormRequest
from ..models import ApimodelUpdateAppResourceRequest
from ..models import ApimodelUpdateAppV2Request
from ..models import ResponseErrorResponse

from ..operations.app_v2 import ApplyAppConfigV2
from ..operations.app_v2 import CreateAppV2
from ..operations.app_v2 import DeleteAppV2
from ..operations.app_v2 import GetAppListV2
from ..operations.app_v2 import GetAppV2
from ..operations.app_v2 import StartAppV2
from ..operations.app_v2 import StopAppV2
from ..operations.app_v2 import UpdateAppResourcesResourceLimitFormV2
from ..operations.app_v2 import UpdateAppResourcesV2
from ..operations.app_v2 import UpdateAppV2


@same_doc_as(ApplyAppConfigV2)
def apply_app_config_v2(
    app: str,
    body: ApimodelApplyAppConfigRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Declaratively create or update an extend app from a spec (ApplyAppConfigV2)

    Idempotent endpoint that creates or updates an Extend app from a declarative spec.
    Uses three-way merge semantics (kubectl apply) for variables, secrets, and permissions.

    Note: `preferred_k8s_namespace` is only used on the initial create call; it is ignored on subsequent apply calls.
    On the first apply after creation the deletion phase is skipped (no prior apply state recorded).

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/apply

        method: POST

        tags: ["App V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelApplyAppConfigRequest in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelAppItem (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        409: Conflict - ResponseErrorResponse (Conflict)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = ApplyAppConfigV2.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ApplyAppConfigV2)
async def apply_app_config_v2_async(
    app: str,
    body: ApimodelApplyAppConfigRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Declaratively create or update an extend app from a spec (ApplyAppConfigV2)

    Idempotent endpoint that creates or updates an Extend app from a declarative spec.
    Uses three-way merge semantics (kubectl apply) for variables, secrets, and permissions.

    Note: `preferred_k8s_namespace` is only used on the initial create call; it is ignored on subsequent apply calls.
    On the first apply after creation the deletion phase is skipped (no prior apply state recorded).

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/apply

        method: POST

        tags: ["App V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelApplyAppConfigRequest in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelAppItem (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        409: Conflict - ResponseErrorResponse (Conflict)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = ApplyAppConfigV2.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(CreateAppV2)
def create_app_v2(
    app: str,
    body: ApimodelCreateAppV2Request,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create new extend app (CreateAppV2)

    [Deprecated] Please use v5 endpoint instead.

    Create new extend app with name provided by {app} path parameter and specified scenario type

    Available scenario:
    - scenario 1: `function-override`
    - scenario 2: `service-extension`
    - scenario 3: `event-handler`


    Available app status:
    - `app-creating`
    - `app-creation-failed`
    - `app-creation-timeout`
    - `app-undeployed`
    - `deployment-in-progress`
    - `deployment-failed`
    - `deployment-timeout`
    - `deployment-running`
    - `deployment-down`
    - `app-stopping`
    - `app-stop-failed`
    - `app-stop-timeout`
    - `app-stopped`
    - `app-removing`
    - `app-removed`
    - `app-remove-timeout`

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}

        method: POST

        tags: ["App V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelCreateAppV2Request in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelAppItem (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        409: Conflict - ResponseErrorResponse (Conflict)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = CreateAppV2.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(CreateAppV2)
async def create_app_v2_async(
    app: str,
    body: ApimodelCreateAppV2Request,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create new extend app (CreateAppV2)

    [Deprecated] Please use v5 endpoint instead.

    Create new extend app with name provided by {app} path parameter and specified scenario type

    Available scenario:
    - scenario 1: `function-override`
    - scenario 2: `service-extension`
    - scenario 3: `event-handler`


    Available app status:
    - `app-creating`
    - `app-creation-failed`
    - `app-creation-timeout`
    - `app-undeployed`
    - `deployment-in-progress`
    - `deployment-failed`
    - `deployment-timeout`
    - `deployment-running`
    - `deployment-down`
    - `app-stopping`
    - `app-stop-failed`
    - `app-stop-timeout`
    - `app-stopped`
    - `app-removing`
    - `app-removed`
    - `app-remove-timeout`

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}

        method: POST

        tags: ["App V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelCreateAppV2Request in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelAppItem (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        409: Conflict - ResponseErrorResponse (Conflict)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = CreateAppV2.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteAppV2)
def delete_app_v2(
    app: str,
    forced: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete extend app by name (DeleteAppV2)

    Delete extend app by given {app} name

    This endpoint will delete app information, configuration, deployments and all related manifest from
    db, storage and cluster

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}

        method: DELETE

        tags: ["App V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        forced: (forced) OPTIONAL str in query

    Responses:
        204: No Content -

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = DeleteAppV2.create(
        app=app,
        forced=forced,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteAppV2)
async def delete_app_v2_async(
    app: str,
    forced: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete extend app by name (DeleteAppV2)

    Delete extend app by given {app} name

    This endpoint will delete app information, configuration, deployments and all related manifest from
    db, storage and cluster

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}

        method: DELETE

        tags: ["App V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        forced: (forced) OPTIONAL str in query

    Responses:
        204: No Content -

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = DeleteAppV2.create(
        app=app,
        forced=forced,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetAppListV2)
def get_app_list_v2(
    body: ApimodelGetAppListV2Request,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get list of extend apps on a given game namespace (GetAppListV2)

    Get list of extend apps on a given game namespace

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps

        method: POST

        tags: ["App V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelGetAppListV2Request in body

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - ApimodelGetAppListV2Response (List of apps)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetAppListV2.create(
        body=body,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetAppListV2)
async def get_app_list_v2_async(
    body: ApimodelGetAppListV2Request,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get list of extend apps on a given game namespace (GetAppListV2)

    Get list of extend apps on a given game namespace

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps

        method: POST

        tags: ["App V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelGetAppListV2Request in body

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - ApimodelGetAppListV2Response (List of apps)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetAppListV2.create(
        body=body,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetAppV2)
def get_app_v2(
    app: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get extend app by name (GetAppV2)

    Get extend app by name

    Available scenario:
    - scenario 1: `function-override`
    - scenario 2: `service-extension`
    - scenario 3: `event-handler`


    Available app status:
    - `app-creating`
    - `app-creation-failed`
    - `app-creation-timeout`
    - `app-undeployed`
    - `deployment-in-progress`
    - `deployment-failed`
    - `deployment-timeout`
    - `deployment-running`
    - `deployment-down`
    - `app-stopping`
    - `app-stop-failed`
    - `app-stop-timeout`
    - `app-stopped`
    - `app-removing`
    - `app-removed`
    - `app-remove-timeout`

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}

        method: GET

        tags: ["App V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelAppItem

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetAppV2.create(
        app=app,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetAppV2)
async def get_app_v2_async(
    app: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get extend app by name (GetAppV2)

    Get extend app by name

    Available scenario:
    - scenario 1: `function-override`
    - scenario 2: `service-extension`
    - scenario 3: `event-handler`


    Available app status:
    - `app-creating`
    - `app-creation-failed`
    - `app-creation-timeout`
    - `app-undeployed`
    - `deployment-in-progress`
    - `deployment-failed`
    - `deployment-timeout`
    - `deployment-running`
    - `deployment-down`
    - `app-stopping`
    - `app-stop-failed`
    - `app-stop-timeout`
    - `app-stopped`
    - `app-removing`
    - `app-removed`
    - `app-remove-timeout`

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}

        method: GET

        tags: ["App V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelAppItem

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetAppV2.create(
        app=app,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(StartAppV2)
def start_app_v2(
    app: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Starts the Application (StartAppV2)

    Starts the Application

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/start

        method: PUT

        tags: ["App V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        202: Accepted - (This means the request to start the application is accepted, and it will be processed in the background.)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = StartAppV2.create(
        app=app,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(StartAppV2)
async def start_app_v2_async(
    app: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Starts the Application (StartAppV2)

    Starts the Application

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/start

        method: PUT

        tags: ["App V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        202: Accepted - (This means the request to start the application is accepted, and it will be processed in the background.)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = StartAppV2.create(
        app=app,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(StopAppV2)
def stop_app_v2(
    app: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Stops the Application (StopAppV2)

    Stops the Application

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/stop

        method: PUT

        tags: ["App V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        202: Accepted - (This means the request to stop the application is accepted, and it will be processed in the background.)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = StopAppV2.create(
        app=app,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(StopAppV2)
async def stop_app_v2_async(
    app: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Stops the Application (StopAppV2)

    Stops the Application

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/stop

        method: PUT

        tags: ["App V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        202: Accepted - (This means the request to stop the application is accepted, and it will be processed in the background.)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = StopAppV2.create(
        app=app,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateAppResourcesResourceLimitFormV2)
def update_app_resources_resource_limit_form_v2(
    app: str,
    body: ApimodelIncreaseLimitFormRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Request Resource Limit to be increased (UpdateAppResourcesResourceLimitFormV2)

    Update app resources provided on request body

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/resources/form

        method: POST

        tags: ["App V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelIncreaseLimitFormRequest in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = UpdateAppResourcesResourceLimitFormV2.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateAppResourcesResourceLimitFormV2)
async def update_app_resources_resource_limit_form_v2_async(
    app: str,
    body: ApimodelIncreaseLimitFormRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Request Resource Limit to be increased (UpdateAppResourcesResourceLimitFormV2)

    Update app resources provided on request body

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/resources/form

        method: POST

        tags: ["App V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelIncreaseLimitFormRequest in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = UpdateAppResourcesResourceLimitFormV2.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateAppResourcesV2)
def update_app_resources_v2(
    app: str,
    body: ApimodelUpdateAppResourceRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update app info (UpdateAppResourcesV2)

    Update app resources provided on request body

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/resources

        method: PATCH

        tags: ["App V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelUpdateAppResourceRequest in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelAppItem (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = UpdateAppResourcesV2.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateAppResourcesV2)
async def update_app_resources_v2_async(
    app: str,
    body: ApimodelUpdateAppResourceRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update app info (UpdateAppResourcesV2)

    Update app resources provided on request body

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/resources

        method: PATCH

        tags: ["App V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelUpdateAppResourceRequest in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelAppItem (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = UpdateAppResourcesV2.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateAppV2)
def update_app_v2(
    app: str,
    body: ApimodelUpdateAppV2Request,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update app info (UpdateAppV2)

    Update app info provided on request body%!(EXTRA string=ADMIN:NAMESPACE:{namespace}:EXTEND:APP, string=UPDATE)

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}

        method: PATCH

        tags: ["App V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelUpdateAppV2Request in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelAppItem (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = UpdateAppV2.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateAppV2)
async def update_app_v2_async(
    app: str,
    body: ApimodelUpdateAppV2Request,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update app info (UpdateAppV2)

    Update app info provided on request body%!(EXTRA string=ADMIN:NAMESPACE:{namespace}:EXTEND:APP, string=UPDATE)

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}

        method: PATCH

        tags: ["App V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelUpdateAppV2Request in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelAppItem (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = UpdateAppV2.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# Custom Service Manager

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.createappparams_autoscaling_request import (
    CreateappparamsAutoscalingRequest,
)
from ..models.createappparams_cpu_request import CreateappparamsCPURequest
from ..models.createappparams_memory_request import CreateappparamsMemoryRequest
from ..models.createappparams_replica_request import CreateappparamsReplicaRequest


class ApimodelCreateAppV5Request(Model):
    """Apimodel create app V5 request (apimodel.CreateAppV5Request)

    Properties:
        scenario: (scenario) REQUIRED str

        autoscaling: (autoscaling) OPTIONAL CreateappparamsAutoscalingRequest

        cpu: (cpu) OPTIONAL CreateappparamsCPURequest

        description: (description) OPTIONAL str

        memory: (memory) OPTIONAL CreateappparamsMemoryRequest

        preferred_k8s_namespace: (preferred_k8s_namespace) OPTIONAL str

        replica: (replica) OPTIONAL CreateappparamsReplicaRequest

        vm_sharing_configuration: (vmSharingConfiguration) OPTIONAL str
    """

    # region fields

    scenario: str  # REQUIRED
    autoscaling: CreateappparamsAutoscalingRequest  # OPTIONAL
    cpu: CreateappparamsCPURequest  # OPTIONAL
    description: str  # OPTIONAL
    memory: CreateappparamsMemoryRequest  # OPTIONAL
    preferred_k8s_namespace: str  # OPTIONAL
    replica: CreateappparamsReplicaRequest  # OPTIONAL
    vm_sharing_configuration: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_scenario(self, value: str) -> ApimodelCreateAppV5Request:
        self.scenario = value
        return self

    def with_autoscaling(
        self, value: CreateappparamsAutoscalingRequest
    ) -> ApimodelCreateAppV5Request:
        self.autoscaling = value
        return self

    def with_cpu(self, value: CreateappparamsCPURequest) -> ApimodelCreateAppV5Request:
        self.cpu = value
        return self

    def with_description(self, value: str) -> ApimodelCreateAppV5Request:
        self.description = value
        return self

    def with_memory(
        self, value: CreateappparamsMemoryRequest
    ) -> ApimodelCreateAppV5Request:
        self.memory = value
        return self

    def with_preferred_k8s_namespace(self, value: str) -> ApimodelCreateAppV5Request:
        self.preferred_k8s_namespace = value
        return self

    def with_replica(
        self, value: CreateappparamsReplicaRequest
    ) -> ApimodelCreateAppV5Request:
        self.replica = value
        return self

    def with_vm_sharing_configuration(self, value: str) -> ApimodelCreateAppV5Request:
        self.vm_sharing_configuration = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "scenario"):
            result["scenario"] = str(self.scenario)
        elif include_empty:
            result["scenario"] = ""
        if hasattr(self, "autoscaling"):
            result["autoscaling"] = self.autoscaling.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["autoscaling"] = CreateappparamsAutoscalingRequest()
        if hasattr(self, "cpu"):
            result["cpu"] = self.cpu.to_dict(include_empty=include_empty)
        elif include_empty:
            result["cpu"] = CreateappparamsCPURequest()
        if hasattr(self, "description"):
            result["description"] = str(self.description)
        elif include_empty:
            result["description"] = ""
        if hasattr(self, "memory"):
            result["memory"] = self.memory.to_dict(include_empty=include_empty)
        elif include_empty:
            result["memory"] = CreateappparamsMemoryRequest()
        if hasattr(self, "preferred_k8s_namespace"):
            result["preferred_k8s_namespace"] = str(self.preferred_k8s_namespace)
        elif include_empty:
            result["preferred_k8s_namespace"] = ""
        if hasattr(self, "replica"):
            result["replica"] = self.replica.to_dict(include_empty=include_empty)
        elif include_empty:
            result["replica"] = CreateappparamsReplicaRequest()
        if hasattr(self, "vm_sharing_configuration"):
            result["vmSharingConfiguration"] = str(self.vm_sharing_configuration)
        elif include_empty:
            result["vmSharingConfiguration"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        scenario: str,
        autoscaling: Optional[CreateappparamsAutoscalingRequest] = None,
        cpu: Optional[CreateappparamsCPURequest] = None,
        description: Optional[str] = None,
        memory: Optional[CreateappparamsMemoryRequest] = None,
        preferred_k8s_namespace: Optional[str] = None,
        replica: Optional[CreateappparamsReplicaRequest] = None,
        vm_sharing_configuration: Optional[str] = None,
        **kwargs,
    ) -> ApimodelCreateAppV5Request:
        instance = cls()
        instance.scenario = scenario
        if autoscaling is not None:
            instance.autoscaling = autoscaling
        if cpu is not None:
            instance.cpu = cpu
        if description is not None:
            instance.description = description
        if memory is not None:
            instance.memory = memory
        if preferred_k8s_namespace is not None:
            instance.preferred_k8s_namespace = preferred_k8s_namespace
        if replica is not None:
            instance.replica = replica
        if vm_sharing_configuration is not None:
            instance.vm_sharing_configuration = vm_sharing_configuration
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelCreateAppV5Request:
        instance = cls()
        if not dict_:
            return instance
        if "scenario" in dict_ and dict_["scenario"] is not None:
            instance.scenario = str(dict_["scenario"])
        elif include_empty:
            instance.scenario = ""
        if "autoscaling" in dict_ and dict_["autoscaling"] is not None:
            instance.autoscaling = CreateappparamsAutoscalingRequest.create_from_dict(
                dict_["autoscaling"], include_empty=include_empty
            )
        elif include_empty:
            instance.autoscaling = CreateappparamsAutoscalingRequest()
        if "cpu" in dict_ and dict_["cpu"] is not None:
            instance.cpu = CreateappparamsCPURequest.create_from_dict(
                dict_["cpu"], include_empty=include_empty
            )
        elif include_empty:
            instance.cpu = CreateappparamsCPURequest()
        if "description" in dict_ and dict_["description"] is not None:
            instance.description = str(dict_["description"])
        elif include_empty:
            instance.description = ""
        if "memory" in dict_ and dict_["memory"] is not None:
            instance.memory = CreateappparamsMemoryRequest.create_from_dict(
                dict_["memory"], include_empty=include_empty
            )
        elif include_empty:
            instance.memory = CreateappparamsMemoryRequest()
        if (
            "preferred_k8s_namespace" in dict_
            and dict_["preferred_k8s_namespace"] is not None
        ):
            instance.preferred_k8s_namespace = str(dict_["preferred_k8s_namespace"])
        elif include_empty:
            instance.preferred_k8s_namespace = ""
        if "replica" in dict_ and dict_["replica"] is not None:
            instance.replica = CreateappparamsReplicaRequest.create_from_dict(
                dict_["replica"], include_empty=include_empty
            )
        elif include_empty:
            instance.replica = CreateappparamsReplicaRequest()
        if (
            "vmSharingConfiguration" in dict_
            and dict_["vmSharingConfiguration"] is not None
        ):
            instance.vm_sharing_configuration = str(dict_["vmSharingConfiguration"])
        elif include_empty:
            instance.vm_sharing_configuration = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelCreateAppV5Request]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelCreateAppV5Request]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelCreateAppV5Request,
        List[ApimodelCreateAppV5Request],
        Dict[Any, ApimodelCreateAppV5Request],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "scenario": "scenario",
            "autoscaling": "autoscaling",
            "cpu": "cpu",
            "description": "description",
            "memory": "memory",
            "preferred_k8s_namespace": "preferred_k8s_namespace",
            "replica": "replica",
            "vmSharingConfiguration": "vm_sharing_configuration",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "scenario": True,
            "autoscaling": False,
            "cpu": False,
            "description": False,
            "memory": False,
            "preferred_k8s_namespace": False,
            "replica": False,
            "vmSharingConfiguration": False,
        }

    # endregion static methods



# Copyright (c) 2025 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.

from dataclasses import dataclass
from logging import Logger
from typing import Optional, Union

from ...core import (
    SDK,
    AccelByteSDK,
    Timer,
)

from ._login_const import (
    DEFAULT_TIMER_INTERVAL,
    DEFAULT_TIMER_REPEATS,
    DEFAULT_TIMER_REPEAT_ON_EXCEPTION,
    DEFAULT_TIMER_AUTOSTART,
    AUTH_SERVICE_LOGGER,
)

from ._refresher import (
    TokenRefresher,
    TokenRefresherOptions,
)

DEFAULT_BACKGROUND_KEY: str = "background_refresh"


@dataclass
class BackgroundOptions(TokenRefresherOptions):
    interval: int = 10


class BackgroundLogin(Timer):
    def __init__(
        self,
        refresher: TokenRefresher,
        /,
        *,
        interval: Optional[Union[int, float]] = None,
        repeats: Optional[int] = None,
        repeat_on_exception: Optional[bool] = None,
        autostart: Optional[bool] = None,
        sdk: Optional[AccelByteSDK] = None,
        logger: Optional[Logger] = None,
        **kwargs,
    ) -> None:
        interval = interval if interval is not None else DEFAULT_TIMER_INTERVAL
        repeats = repeats if repeats is not None else DEFAULT_TIMER_REPEATS
        repeat_on_exception = (
            repeat_on_exception
            if repeat_on_exception is not None
            else DEFAULT_TIMER_REPEAT_ON_EXCEPTION
        )
        autostart = autostart if autostart is not None else DEFAULT_TIMER_AUTOSTART

        sdk = sdk or SDK
        logger = logger or AUTH_SERVICE_LOGGER

        self.refresher = refresher
        self.sdk = sdk
        self.logger = logger

        super().__init__(
            interval,
            self.run,
            daemon=True,
            repeats=repeats,
            autostart=autostart,
            repeat_on_exception=repeat_on_exception,
            logger=self.logger,
        )

    def run(self):
        self.refresher.refresh(sdk=self.sdk)

    async def run_async(self):
        await self.refresher.refresh_async(sdk=self.sdk)


def enable_background_refresh(
    sdk: AccelByteSDK,
    options: Optional[BackgroundOptions] = None,
    key: str = DEFAULT_BACKGROUND_KEY,
    **kwargs,
) -> None:
    options = options or BackgroundOptions()
    refresher = TokenRefresher(options=options)
    background = BackgroundLogin(
        refresher,
        interval=options.interval,
        repeats=-1,
        repeat_on_exception=True,
        autostart=True,
        sdk=sdk,
    )

    disable_background_refresh(sdk=sdk, key=key, **kwargs)

    setattr(sdk, key, background)


def disable_background_refresh(
    sdk: AccelByteSDK, key: str = DEFAULT_BACKGROUND_KEY, **kwargs
) -> None:
    if existing := getattr(sdk, key, None):
        if existing_cancel := getattr(existing, "cancel", None):
            if callable(existing_cancel):
                existing_cancel()


__all__ = [
    "BackgroundLogin",
    "BackgroundOptions",
    "enable_background_refresh",
    "disable_background_refresh",
]



import calendar
import datetime
import random
import string
import urllib.parse

from typing import Any, List, Optional, Tuple, Union

ADJECTIVE_ED = [
    "Abandoned",
    "Abdicated",
    "Aged",
    "Bamboozled",
    "Bemused",
    "Broken",
    "Caged",
    "Charged",
    "Confined",
    "Defamed",
    "Defeated",
    "Distorted",
    "Edited",
    "Enhanced",
    "Enraged",
]

ADJECTIVE_IC = [
    "Archaic",
    "Athletic",
    "Authentic",
    "Ballistic",
    "Basic",
    "Bionic",
    "Caloric",
    "Climatic",
    "Cubic",
    "Democratic",
    "Demonic",
    "Draconic",
    "Eccentric",
    "Elastic",
    "Energetic",
]

ADJECTIVE_OUS = [
    "Ambidextrous",
    "Amphibious",
    "Autonomous",
    "Blasphemous",
    "Boisterous",
    "Bumptious",
    "Carnivorous",
    "Clamorous",
    "Copious",
    "Dangerous",
    "Devious",
    "Dexterous",
    "Egregious",
    "Envious",
    "Euphonious",
]

CREATURE_ANIMAL = [
    "Aardvark",
    "Antelope",
    "Axolotl",
    "Baboon",
    "Badger",
    "Beetle",
    "Camel",
    "Carp",
    "Chipmunk",
    "Deer",
    "Dingo",
    "Dodo",
    "Eagle",
    "Elephant",
    "Emu",
]

COUNTRY_2 = [
    "CA",
    "CN",
    "GB",
    "ID",
    "IE",
    "PH",
    "RU",
    "US",
]

FOOD_DESSERT = [
    "Apple Pie",
    "Apple Strudel",
    "Banana Bread",
    "Birthday Cake",
    "Blueberry Muffin" "Cheesecake",
    "Cherry Pie",
    "Cinnamon Rolls",
    "Dacquoise",
    "Doughnut",
    "Dundee Cake",
    "Fudge",
    "Funnel Cake",
    "Frozen Yogurt",
]

FOOD_FRUIT_VEGGIE = [
    "Apple",
    "Apricot",
    "Avocado",
    "Banana",
    "Blackberry",
    "Blueberry",
    "Cantaloupe",
    "Cherry",
    "Coconut",
    "Date",
    "Dill",
    "Durian",
    "Eggplant",
    "Elderberry",
    "Etrog",
]

NAME_FIRST = [
    "Adam",
    "Alice",
    "Bob",
    "Bonnie",
    "Clyde",
    "Claire",
    "Dan",
    "Diana",
    "Earl",
    "Eve",
]

NAME_LAST = [
    "Anderson",
    "Allen",
    "Adams",
    "Brown",
    "Baker",
    "Butler",
    "Clark",
    "Cook",
    "Cooper",
    "Davis",
    "Dunn",
    "Dixon",
    "Evans",
    "Edwards",
    "Elliott",
]

SPECIAL_CHARACTERS = " `~!@#$%^&*()-_=+[{]}|:;<,>.?"


def random_adjective(**kwargs) -> str:
    result = random.choice([*ADJECTIVE_ED, *ADJECTIVE_IC, *ADJECTIVE_OUS])
    return result


def random_adult_birthdate(
    fmt: str = "%Y-%m-%d", legal_age_range: Optional[Union[int, Tuple[int, int]]] = None
) -> str:
    if isinstance(legal_age_range, int):
        legal_age = legal_age_range
    else:
        legal_age_min, legal_age_max = legal_age_range if legal_age_range else (21, 50)
        legal_age = random.randint(legal_age_min, legal_age_max)
    delta = legal_age * 366
    delta_bonus = random.randint(0, 366)
    today = datetime.date.today()
    birthdate = today - datetime.timedelta(days=delta + delta_bonus)
    result = birthdate.strftime(fmt)
    return result


def random_country(**kwargs) -> str:
    result = random.choice(COUNTRY_2)
    return result


def random_creature(**kwargs) -> str:
    result = random.choice(CREATURE_ANIMAL)
    return result


def random_date(
    year: Optional[int] = None,
    month: Optional[int] = None,
    day: Optional[int] = None,
    fmt: str = "%Y-%m-%d",
) -> str:
    year = year or random.randint(1970, datetime.date.today().year)
    month = month or random.randint(1, 12)
    day = day or random.randint(1, calendar.monthrange(year, month)[1])
    date = datetime.date(year=year, month=month, day=day)
    result = date.strftime(fmt)
    return result


def random_email(
    username: Optional[str] = None,
    domain_name: Optional[str] = None,
    domain: Optional[str] = None,
) -> str:
    username = (
        username
        or (
            random_adjective()
            + random_thing()
            + "+"
            + random_string(6, [*string.hexdigits])
        ).upper()
    )
    username = username.replace(" ", "")
    domain_name = domain_name or "fakemail"
    domain = domain or "com"
    result = f"{username}@{domain_name}.{domain}"
    return result


def random_first_name(**kwargs) -> str:
    result = random.choice(NAME_FIRST)
    return result


def random_food(**kwargs) -> str:
    result = random.choice([*FOOD_DESSERT, *FOOD_FRUIT_VEGGIE])
    return result


def random_int(min_val: int = 0, max_val: int = 100) -> int:
    result = random.randint(min_val, max_val)
    return result


def random_last_name(**kwargs) -> str:
    result = random.choice(NAME_LAST)
    return result


def random_string(length, choice: List[str] = None) -> str:
    choice = choice if choice else [*string.ascii_letters]
    result = "".join(random.choice(choice) for _ in range(length))
    return result


def random_thing(**kwargs) -> str:
    result = random.choice([*CREATURE_ANIMAL, *FOOD_DESSERT, *FOOD_FRUIT_VEGGIE])
    return result


def random_password(length: int = 16) -> str:
    result = random_string(
        length, [*string.ascii_letters, *string.digits, *SPECIAL_CHARACTERS]
    )
    return result


def random_uid(**kwargs) -> str:
    result = random_string(32, [*string.hexdigits]).lower()
    return result


def random_url(file_ext: Optional[str] = None) -> str:
    file_ext = f".{file_ext.removeprefix('.')}" if file_ext else ""
    result = (
        f"https://www.example.com/{urllib.parse.quote(random_string(10))}{file_ext}"
    )
    return result


def random_username(**kwargs) -> str:
    result = (
        random_adjective()
        + random_adjective()
        + random_creature()
        + random_string(6, [*string.hexdigits]).upper()
    )
    result = result.replace(" ", "")
    return result


def random_version(
    major: Optional[int] = None,
    minor: Optional[int] = None,
    patch: Optional[int] = None,
    prefix: Optional[str] = None,
    suffix: Optional[str] = None,
) -> str:
    major = major or random.randint(0, 99)
    minor = minor or random.randint(0, 99)
    patch = patch or random.randint(0, 99)
    prefix = prefix or ""
    suffix = suffix or ""
    result = f"{prefix}{major}.{minor}.{patch}{suffix}"
    return result


def random_zip_code(length: int = 5) -> str:
    result = random_string(length, [*string.digits])
    return result


def randomize(type_: Optional[str] = None, **kwargs) -> Any:
    types = {
        "adjective": random_adjective,
        "adult_birthdate": random_adult_birthdate,
        "country": random_country,
        "date": random_date,
        "creature": random_creature,
        "email": random_email,
        "first_name": random_first_name,
        "food": random_food,
        "int": random_int,
        "last_name": random_last_name,
        "password": random_password,
        "slug": random_username,
        "thing": random_thing,
        "uid": random_uid,
        "url": random_url,
        "version": random_version,
        "zip_code": random_zip_code,
    }
    func = types.get(type_)
    if func is not None:
        return func(**kwargs)
    return random_string(length=12)



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ModelCheckAvailabilityResponse
from ..models import ModelPlatformDefaultsResponse
from ..models import ModelPlatformDomainDeleteRequest
from ..models import ModelPlatformDomainPatchRequest
from ..models import ModelPlatformDomainResponse
from ..models import ModelPlatformDomainUpdateRequest
from ..models import ModelPublicThirdPartyPlatformInfo
from ..models import ModelThirdPartyLoginPlatformCredentialRequest
from ..models import ModelThirdPartyLoginPlatformCredentialResponse
from ..models import RestErrorResponse

from ..operations.third_party_credential import AddThirdPartyLoginPlatformCredentialV3
from ..operations.third_party_credential import (
    AdminCheckThirdPartyLoginPlatformAvailabilityV3,
)
from ..operations.third_party_credential import (
    DeleteThirdPartyLoginPlatformCredentialV3,
)
from ..operations.third_party_credential import DeleteThirdPartyLoginPlatformDomainV3
from ..operations.third_party_credential import GetThirdPartyLoginPlatformDefaultsV3
from ..operations.third_party_credential import (
    PartialUpdateThirdPartyLoginPlatformDomainV3,
)
from ..operations.third_party_credential import RetrieveActiveOIDCClientsPublicV3
from ..operations.third_party_credential import (
    RetrieveAllActiveThirdPartyLoginPlatformCredentialPublicV3,
)
from ..operations.third_party_credential import (
    RetrieveAllActiveThirdPartyLoginPlatformCredentialV3,
)
from ..operations.third_party_credential import (
    RetrieveAllThirdPartyLoginPlatformCredentialV3,
)
from ..operations.third_party_credential import (
    RetrieveThirdPartyLoginPlatformCredentialV3,
)
from ..operations.third_party_credential import (
    UpdateThirdPartyLoginPlatformCredentialV3,
)
from ..operations.third_party_credential import UpdateThirdPartyLoginPlatformDomainV3


@same_doc_as(AddThirdPartyLoginPlatformCredentialV3)
def add_third_party_login_platform_credential_v3(
    body: ModelThirdPartyLoginPlatformCredentialRequest,
    platform_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Add Third Party Platform Credential (AddThirdPartyLoginPlatformCredentialV3)

    Creates a third-party platform credential. Configure the Secret field according to the target platform (see field description). For generic OAuth flow, set GenericOauthFlow to true and configure the corresponding fields.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/platforms/{platformId}/clients

        method: POST

        tags: ["Third Party Credential"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelThirdPartyLoginPlatformCredentialRequest in body

        namespace: (namespace) REQUIRED str in path

        platform_id: (platformId) REQUIRED str in path

    Responses:
        201: Created - ModelThirdPartyLoginPlatformCredentialResponse (Third Party Credential Created)

        400: Bad Request - RestErrorResponse (20019: unable to parse request body)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AddThirdPartyLoginPlatformCredentialV3.create(
        body=body,
        platform_id=platform_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AddThirdPartyLoginPlatformCredentialV3)
async def add_third_party_login_platform_credential_v3_async(
    body: ModelThirdPartyLoginPlatformCredentialRequest,
    platform_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Add Third Party Platform Credential (AddThirdPartyLoginPlatformCredentialV3)

    Creates a third-party platform credential. Configure the Secret field according to the target platform (see field description). For generic OAuth flow, set GenericOauthFlow to true and configure the corresponding fields.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/platforms/{platformId}/clients

        method: POST

        tags: ["Third Party Credential"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelThirdPartyLoginPlatformCredentialRequest in body

        namespace: (namespace) REQUIRED str in path

        platform_id: (platformId) REQUIRED str in path

    Responses:
        201: Created - ModelThirdPartyLoginPlatformCredentialResponse (Third Party Credential Created)

        400: Bad Request - RestErrorResponse (20019: unable to parse request body)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AddThirdPartyLoginPlatformCredentialV3.create(
        body=body,
        platform_id=platform_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminCheckThirdPartyLoginPlatformAvailabilityV3)
def admin_check_third_party_login_platform_availability_v3(
    platform_id: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Check 3rd party platform availability (AdminCheckThirdPartyLoginPlatformAvailabilityV3)

    This is the API to check specific 3rd party platform availability.
    Passing platform group name or it's member will return same platform availability data
    Supported third party platform and platform group:
    - PSN group(psn)
    - ps4web
    - ps4
    - ps5

    Properties:
        url: /iam/v3/admin/platforms/{platformId}/availability

        method: GET

        tags: ["Third Party Credential"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        platform_id: (platformId) REQUIRED str in path

    Responses:
        200: OK - ModelCheckAvailabilityResponse (Platform Availability Check Retrieved)

        400: Bad Request - RestErrorResponse (10226: third party platform is not supported)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)
    """
    request = AdminCheckThirdPartyLoginPlatformAvailabilityV3.create(
        platform_id=platform_id,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminCheckThirdPartyLoginPlatformAvailabilityV3)
async def admin_check_third_party_login_platform_availability_v3_async(
    platform_id: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Check 3rd party platform availability (AdminCheckThirdPartyLoginPlatformAvailabilityV3)

    This is the API to check specific 3rd party platform availability.
    Passing platform group name or it's member will return same platform availability data
    Supported third party platform and platform group:
    - PSN group(psn)
    - ps4web
    - ps4
    - ps5

    Properties:
        url: /iam/v3/admin/platforms/{platformId}/availability

        method: GET

        tags: ["Third Party Credential"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        platform_id: (platformId) REQUIRED str in path

    Responses:
        200: OK - ModelCheckAvailabilityResponse (Platform Availability Check Retrieved)

        400: Bad Request - RestErrorResponse (10226: third party platform is not supported)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)
    """
    request = AdminCheckThirdPartyLoginPlatformAvailabilityV3.create(
        platform_id=platform_id,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteThirdPartyLoginPlatformCredentialV3)
def delete_third_party_login_platform_credential_v3(
    platform_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Third Party Platform Credential (DeleteThirdPartyLoginPlatformCredentialV3)

    Deletes the third-party platform credential for the specified platform.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/platforms/{platformId}/clients

        method: DELETE

        tags: ["Third Party Credential"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        platform_id: (platformId) REQUIRED str in path

    Responses:
        204: No Content - (Delete Process Successful)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10175: third party credential not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = DeleteThirdPartyLoginPlatformCredentialV3.create(
        platform_id=platform_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteThirdPartyLoginPlatformCredentialV3)
async def delete_third_party_login_platform_credential_v3_async(
    platform_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Third Party Platform Credential (DeleteThirdPartyLoginPlatformCredentialV3)

    Deletes the third-party platform credential for the specified platform.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/platforms/{platformId}/clients

        method: DELETE

        tags: ["Third Party Credential"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        platform_id: (platformId) REQUIRED str in path

    Responses:
        204: No Content - (Delete Process Successful)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10175: third party credential not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = DeleteThirdPartyLoginPlatformCredentialV3.create(
        platform_id=platform_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteThirdPartyLoginPlatformDomainV3)
def delete_third_party_login_platform_domain_v3(
    body: ModelPlatformDomainDeleteRequest,
    platform_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Unregister Third Party Platform Credential Domain (DeleteThirdPartyLoginPlatformDomainV3)

    This is the API to unregister 3rd Platform domain.
    If there is a ssoGroups in request body, then this request wil only delete the sso group from the target domain, it will not delete domain.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/platforms/{platformId}/clients/domain

        method: DELETE

        tags: ["Third Party Credential"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelPlatformDomainDeleteRequest in body

        namespace: (namespace) REQUIRED str in path

        platform_id: (platformId) REQUIRED str in path

    Responses:
        204: No Content - (Domain unregister succeeded)

        400: Bad Request - RestErrorResponse (Bad Request)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10175: third party credential not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = DeleteThirdPartyLoginPlatformDomainV3.create(
        body=body,
        platform_id=platform_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteThirdPartyLoginPlatformDomainV3)
async def delete_third_party_login_platform_domain_v3_async(
    body: ModelPlatformDomainDeleteRequest,
    platform_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Unregister Third Party Platform Credential Domain (DeleteThirdPartyLoginPlatformDomainV3)

    This is the API to unregister 3rd Platform domain.
    If there is a ssoGroups in request body, then this request wil only delete the sso group from the target domain, it will not delete domain.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/platforms/{platformId}/clients/domain

        method: DELETE

        tags: ["Third Party Credential"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelPlatformDomainDeleteRequest in body

        namespace: (namespace) REQUIRED str in path

        platform_id: (platformId) REQUIRED str in path

    Responses:
        204: No Content - (Domain unregister succeeded)

        400: Bad Request - RestErrorResponse (Bad Request)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10175: third party credential not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = DeleteThirdPartyLoginPlatformDomainV3.create(
        body=body,
        platform_id=platform_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetThirdPartyLoginPlatformDefaultsV3)
def get_third_party_login_platform_defaults_v3(
    platform_id: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Get Third Party Platform Defaults (GetThirdPartyLoginPlatformDefaultsV3)

    Returns the built-in default configuration values for a given platform.

    Properties:
        url: /iam/v3/admin/platforms/{platformId}/defaults

        method: GET

        tags: ["Third Party Credential"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        platform_id: (platformId) REQUIRED str in path

    Responses:
        200: OK - ModelPlatformDefaultsResponse (Platform defaults retrieved)
    """
    request = GetThirdPartyLoginPlatformDefaultsV3.create(
        platform_id=platform_id,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetThirdPartyLoginPlatformDefaultsV3)
async def get_third_party_login_platform_defaults_v3_async(
    platform_id: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Get Third Party Platform Defaults (GetThirdPartyLoginPlatformDefaultsV3)

    Returns the built-in default configuration values for a given platform.

    Properties:
        url: /iam/v3/admin/platforms/{platformId}/defaults

        method: GET

        tags: ["Third Party Credential"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        platform_id: (platformId) REQUIRED str in path

    Responses:
        200: OK - ModelPlatformDefaultsResponse (Platform defaults retrieved)
    """
    request = GetThirdPartyLoginPlatformDefaultsV3.create(
        platform_id=platform_id,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PartialUpdateThirdPartyLoginPlatformDomainV3)
def partial_update_third_party_login_platform_domain_v3(
    body: ModelPlatformDomainPatchRequest,
    platform_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Partial Update Third Party Platform Credential Domain (PartialUpdateThirdPartyLoginPlatformDomainV3)

    This is the API to patch update 3rd Platform domain.
    This API is a create or partial-update behavior. If it is update, it is a partial update behavior.

    **Request body:** Partial domain configuration update. Only provided fields will be updated.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/platforms/{platformId}/clients/domain

        method: PATCH

        tags: ["Third Party Credential"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelPlatformDomainPatchRequest in body

        namespace: (namespace) REQUIRED str in path

        platform_id: (platformId) REQUIRED str in path

    Responses:
        200: OK - ModelPlatformDomainResponse (Domain config partial updated)

        400: Bad Request - RestErrorResponse (Bad Request)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10175: third party credential not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = PartialUpdateThirdPartyLoginPlatformDomainV3.create(
        body=body,
        platform_id=platform_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PartialUpdateThirdPartyLoginPlatformDomainV3)
async def partial_update_third_party_login_platform_domain_v3_async(
    body: ModelPlatformDomainPatchRequest,
    platform_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Partial Update Third Party Platform Credential Domain (PartialUpdateThirdPartyLoginPlatformDomainV3)

    This is the API to patch update 3rd Platform domain.
    This API is a create or partial-update behavior. If it is update, it is a partial update behavior.

    **Request body:** Partial domain configuration update. Only provided fields will be updated.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/platforms/{platformId}/clients/domain

        method: PATCH

        tags: ["Third Party Credential"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelPlatformDomainPatchRequest in body

        namespace: (namespace) REQUIRED str in path

        platform_id: (platformId) REQUIRED str in path

    Responses:
        200: OK - ModelPlatformDomainResponse (Domain config partial updated)

        400: Bad Request - RestErrorResponse (Bad Request)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10175: third party credential not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = PartialUpdateThirdPartyLoginPlatformDomainV3.create(
        body=body,
        platform_id=platform_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RetrieveActiveOIDCClientsPublicV3)
def retrieve_active_oidc_clients_public_v3(
    client_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Active OIDC Platform Credentials by Client ID (RetrieveActiveOIDCClientsPublicV3)

    Retrieves all active OIDC platform credentials filtered by client ID.

    Properties:
        url: /iam/v3/public/namespaces/{namespace}/platforms/clients/oidc

        method: GET

        tags: ["Third Party Credential"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        client_id: (clientId) REQUIRED str in query

    Responses:
        200: OK - List[ModelPublicThirdPartyPlatformInfo] (All Active OIDC Credential Retrieved)

        404: Not Found - RestErrorResponse (10175: third party credential not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RetrieveActiveOIDCClientsPublicV3.create(
        client_id=client_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RetrieveActiveOIDCClientsPublicV3)
async def retrieve_active_oidc_clients_public_v3_async(
    client_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Active OIDC Platform Credentials by Client ID (RetrieveActiveOIDCClientsPublicV3)

    Retrieves all active OIDC platform credentials filtered by client ID.

    Properties:
        url: /iam/v3/public/namespaces/{namespace}/platforms/clients/oidc

        method: GET

        tags: ["Third Party Credential"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        client_id: (clientId) REQUIRED str in query

    Responses:
        200: OK - List[ModelPublicThirdPartyPlatformInfo] (All Active OIDC Credential Retrieved)

        404: Not Found - RestErrorResponse (10175: third party credential not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RetrieveActiveOIDCClientsPublicV3.create(
        client_id=client_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RetrieveAllActiveThirdPartyLoginPlatformCredentialPublicV3)
def retrieve_all_active_third_party_login_platform_credential_public_v3(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get All Active Third Party Platform Credentials (RetrieveAllActiveThirdPartyLoginPlatformCredentialPublicV3)

    Retrieves all active third-party platform credentials for the namespace.

    Properties:
        url: /iam/v3/public/namespaces/{namespace}/platforms/clients/active

        method: GET

        tags: ["Third Party Credential"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - List[ModelPublicThirdPartyPlatformInfo] (All Active Third Party Credential Retrieved)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10175: third party credential not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RetrieveAllActiveThirdPartyLoginPlatformCredentialPublicV3.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RetrieveAllActiveThirdPartyLoginPlatformCredentialPublicV3)
async def retrieve_all_active_third_party_login_platform_credential_public_v3_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get All Active Third Party Platform Credentials (RetrieveAllActiveThirdPartyLoginPlatformCredentialPublicV3)

    Retrieves all active third-party platform credentials for the namespace.

    Properties:
        url: /iam/v3/public/namespaces/{namespace}/platforms/clients/active

        method: GET

        tags: ["Third Party Credential"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - List[ModelPublicThirdPartyPlatformInfo] (All Active Third Party Credential Retrieved)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10175: third party credential not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RetrieveAllActiveThirdPartyLoginPlatformCredentialPublicV3.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RetrieveAllActiveThirdPartyLoginPlatformCredentialV3)
def retrieve_all_active_third_party_login_platform_credential_v3(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get All Active Third Party Platform Active Credential (RetrieveAllActiveThirdPartyLoginPlatformCredentialV3)

    Retrieves all active third-party platform credentials.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/platforms/all/clients/active

        method: GET

        tags: ["Third Party Credential"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - List[ModelThirdPartyLoginPlatformCredentialResponse] (All Active Third Party Credential Retrieved)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10175: third party credential not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RetrieveAllActiveThirdPartyLoginPlatformCredentialV3.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RetrieveAllActiveThirdPartyLoginPlatformCredentialV3)
async def retrieve_all_active_third_party_login_platform_credential_v3_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get All Active Third Party Platform Active Credential (RetrieveAllActiveThirdPartyLoginPlatformCredentialV3)

    Retrieves all active third-party platform credentials.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/platforms/all/clients/active

        method: GET

        tags: ["Third Party Credential"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - List[ModelThirdPartyLoginPlatformCredentialResponse] (All Active Third Party Credential Retrieved)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10175: third party credential not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RetrieveAllActiveThirdPartyLoginPlatformCredentialV3.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RetrieveAllThirdPartyLoginPlatformCredentialV3)
def retrieve_all_third_party_login_platform_credential_v3(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get All Third Party Platform Credential (RetrieveAllThirdPartyLoginPlatformCredentialV3)

    Retrieves all third-party platform credentials.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/platforms/all/clients

        method: GET

        tags: ["Third Party Credential"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - List[ModelThirdPartyLoginPlatformCredentialResponse] (All Active Third Party Credential Retrieved)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10175: third party credential not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RetrieveAllThirdPartyLoginPlatformCredentialV3.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RetrieveAllThirdPartyLoginPlatformCredentialV3)
async def retrieve_all_third_party_login_platform_credential_v3_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get All Third Party Platform Credential (RetrieveAllThirdPartyLoginPlatformCredentialV3)

    Retrieves all third-party platform credentials.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/platforms/all/clients

        method: GET

        tags: ["Third Party Credential"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - List[ModelThirdPartyLoginPlatformCredentialResponse] (All Active Third Party Credential Retrieved)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10175: third party credential not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RetrieveAllThirdPartyLoginPlatformCredentialV3.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RetrieveThirdPartyLoginPlatformCredentialV3)
def retrieve_third_party_login_platform_credential_v3(
    platform_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve Third Party Platform Credential (RetrieveThirdPartyLoginPlatformCredentialV3)

    Retrieves the third-party platform credential for the specified platform.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/platforms/{platformId}/clients

        method: GET

        tags: ["Third Party Credential"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        platform_id: (platformId) REQUIRED str in path

    Responses:
        200: OK - ModelThirdPartyLoginPlatformCredentialResponse (Third Party Credential Found)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10175: third party credential not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RetrieveThirdPartyLoginPlatformCredentialV3.create(
        platform_id=platform_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RetrieveThirdPartyLoginPlatformCredentialV3)
async def retrieve_third_party_login_platform_credential_v3_async(
    platform_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve Third Party Platform Credential (RetrieveThirdPartyLoginPlatformCredentialV3)

    Retrieves the third-party platform credential for the specified platform.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/platforms/{platformId}/clients

        method: GET

        tags: ["Third Party Credential"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        platform_id: (platformId) REQUIRED str in path

    Responses:
        200: OK - ModelThirdPartyLoginPlatformCredentialResponse (Third Party Credential Found)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10175: third party credential not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RetrieveThirdPartyLoginPlatformCredentialV3.create(
        platform_id=platform_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateThirdPartyLoginPlatformCredentialV3)
def update_third_party_login_platform_credential_v3(
    body: ModelThirdPartyLoginPlatformCredentialRequest,
    platform_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Third Party Platform Credential (UpdateThirdPartyLoginPlatformCredentialV3)

    Updates a third-party platform credential. Configure the Secret field according to the target platform (see field description). For generic OAuth flow, set GenericOauthFlow to true and configure the corresponding fields.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/platforms/{platformId}/clients

        method: PATCH

        tags: ["Third Party Credential"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelThirdPartyLoginPlatformCredentialRequest in body

        namespace: (namespace) REQUIRED str in path

        platform_id: (platformId) REQUIRED str in path

    Responses:
        200: OK - ModelThirdPartyLoginPlatformCredentialResponse (OK)

        400: Bad Request - RestErrorResponse (Bad Request)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10175: third party credential not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = UpdateThirdPartyLoginPlatformCredentialV3.create(
        body=body,
        platform_id=platform_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateThirdPartyLoginPlatformCredentialV3)
async def update_third_party_login_platform_credential_v3_async(
    body: ModelThirdPartyLoginPlatformCredentialRequest,
    platform_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Third Party Platform Credential (UpdateThirdPartyLoginPlatformCredentialV3)

    Updates a third-party platform credential. Configure the Secret field according to the target platform (see field description). For generic OAuth flow, set GenericOauthFlow to true and configure the corresponding fields.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/platforms/{platformId}/clients

        method: PATCH

        tags: ["Third Party Credential"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelThirdPartyLoginPlatformCredentialRequest in body

        namespace: (namespace) REQUIRED str in path

        platform_id: (platformId) REQUIRED str in path

    Responses:
        200: OK - ModelThirdPartyLoginPlatformCredentialResponse (OK)

        400: Bad Request - RestErrorResponse (Bad Request)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10175: third party credential not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = UpdateThirdPartyLoginPlatformCredentialV3.create(
        body=body,
        platform_id=platform_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateThirdPartyLoginPlatformDomainV3)
def update_third_party_login_platform_domain_v3(
    body: ModelPlatformDomainUpdateRequest,
    platform_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Set Third Party Platform Credential Domain (UpdateThirdPartyLoginPlatformDomainV3)

    This is the API to set 3rd Platform domain.
    This API is a create-or-update behavior. If it is update, it is a replacement behavior.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/platforms/{platformId}/clients/domain

        method: PUT

        tags: ["Third Party Credential"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelPlatformDomainUpdateRequest in body

        namespace: (namespace) REQUIRED str in path

        platform_id: (platformId) REQUIRED str in path

    Responses:
        200: OK - ModelPlatformDomainResponse (Domain config updated)

        400: Bad Request - RestErrorResponse (Bad Request)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10175: third party credential not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = UpdateThirdPartyLoginPlatformDomainV3.create(
        body=body,
        platform_id=platform_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateThirdPartyLoginPlatformDomainV3)
async def update_third_party_login_platform_domain_v3_async(
    body: ModelPlatformDomainUpdateRequest,
    platform_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Set Third Party Platform Credential Domain (UpdateThirdPartyLoginPlatformDomainV3)

    This is the API to set 3rd Platform domain.
    This API is a create-or-update behavior. If it is update, it is a replacement behavior.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/platforms/{platformId}/clients/domain

        method: PUT

        tags: ["Third Party Credential"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelPlatformDomainUpdateRequest in body

        namespace: (namespace) REQUIRED str in path

        platform_id: (platformId) REQUIRED str in path

    Responses:
        200: OK - ModelPlatformDomainResponse (Domain config updated)

        400: Bad Request - RestErrorResponse (Bad Request)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10175: third party credential not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = UpdateThirdPartyLoginPlatformDomainV3.create(
        body=body,
        platform_id=platform_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# Fleet Commander

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ApiFleetServerConnectionInfoResponse(Model):
    """Api fleet server connection info response (api.FleetServerConnectionInfoResponse)

    Properties:
        expires_at: (expiresAt) REQUIRED str

        host: (host) REQUIRED str

        logstream_port: (logstreamPort) REQUIRED int

        secret: (secret) REQUIRED str
    """

    # region fields

    expires_at: str  # REQUIRED
    host: str  # REQUIRED
    logstream_port: int  # REQUIRED
    secret: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_expires_at(self, value: str) -> ApiFleetServerConnectionInfoResponse:
        self.expires_at = value
        return self

    def with_host(self, value: str) -> ApiFleetServerConnectionInfoResponse:
        self.host = value
        return self

    def with_logstream_port(self, value: int) -> ApiFleetServerConnectionInfoResponse:
        self.logstream_port = value
        return self

    def with_secret(self, value: str) -> ApiFleetServerConnectionInfoResponse:
        self.secret = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "expires_at"):
            result["expiresAt"] = str(self.expires_at)
        elif include_empty:
            result["expiresAt"] = ""
        if hasattr(self, "host"):
            result["host"] = str(self.host)
        elif include_empty:
            result["host"] = ""
        if hasattr(self, "logstream_port"):
            result["logstreamPort"] = int(self.logstream_port)
        elif include_empty:
            result["logstreamPort"] = 0
        if hasattr(self, "secret"):
            result["secret"] = str(self.secret)
        elif include_empty:
            result["secret"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls, expires_at: str, host: str, logstream_port: int, secret: str, **kwargs
    ) -> ApiFleetServerConnectionInfoResponse:
        instance = cls()
        instance.expires_at = expires_at
        instance.host = host
        instance.logstream_port = logstream_port
        instance.secret = secret
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApiFleetServerConnectionInfoResponse:
        instance = cls()
        if not dict_:
            return instance
        if "expiresAt" in dict_ and dict_["expiresAt"] is not None:
            instance.expires_at = str(dict_["expiresAt"])
        elif include_empty:
            instance.expires_at = ""
        if "host" in dict_ and dict_["host"] is not None:
            instance.host = str(dict_["host"])
        elif include_empty:
            instance.host = ""
        if "logstreamPort" in dict_ and dict_["logstreamPort"] is not None:
            instance.logstream_port = int(dict_["logstreamPort"])
        elif include_empty:
            instance.logstream_port = 0
        if "secret" in dict_ and dict_["secret"] is not None:
            instance.secret = str(dict_["secret"])
        elif include_empty:
            instance.secret = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApiFleetServerConnectionInfoResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApiFleetServerConnectionInfoResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApiFleetServerConnectionInfoResponse,
        List[ApiFleetServerConnectionInfoResponse],
        Dict[Any, ApiFleetServerConnectionInfoResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "expiresAt": "expires_at",
            "host": "host",
            "logstreamPort": "logstream_port",
            "secret": "secret",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "expiresAt": True,
            "host": True,
            "logstreamPort": True,
            "secret": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ApimodelCreateSubscriptionRequest
from ..models import ApimodelCreateSubscriptionResponse
from ..models import ApimodelCreateTopicRequest
from ..models import ApimodelListTopicsResponse
from ..models import ApimodelTopicResponse
from ..models import ApimodelTopicSubscription
from ..models import ResponseErrorResponse

from ..operations.async_messaging import CreateSubscriptionHandler
from ..operations.async_messaging import CreateTopicHandler
from ..operations.async_messaging import DeleteTopicHandler
from ..operations.async_messaging import ListTopicsHandler
from ..operations.async_messaging import UnsubscribeTopicHandler


@same_doc_as(CreateSubscriptionHandler)
def create_subscription_handler(
    app: str,
    body: ApimodelCreateSubscriptionRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Subscribe To Topic (CreateSubscriptionHandler)

    Subscribe App queue to given Topic name list inside a game namespace.

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/asyncmessaging/topics/subscriptions

        method: POST

        tags: ["Async Messaging"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelCreateSubscriptionRequest in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelCreateSubscriptionResponse (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = CreateSubscriptionHandler.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateSubscriptionHandler)
async def create_subscription_handler_async(
    app: str,
    body: ApimodelCreateSubscriptionRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Subscribe To Topic (CreateSubscriptionHandler)

    Subscribe App queue to given Topic name list inside a game namespace.

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/asyncmessaging/topics/subscriptions

        method: POST

        tags: ["Async Messaging"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelCreateSubscriptionRequest in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelCreateSubscriptionResponse (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = CreateSubscriptionHandler.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(CreateTopicHandler)
def create_topic_handler(
    body: ApimodelCreateTopicRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create Async Messaging Topic (CreateTopicHandler)

    Create New Async Messaging Topic, the topic name is unique inside a game namespace.

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/asyncmessaging/topics

        method: POST

        tags: ["Async Messaging"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelCreateTopicRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelTopicResponse (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        409: Conflict - ResponseErrorResponse (Conflict)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = CreateTopicHandler.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateTopicHandler)
async def create_topic_handler_async(
    body: ApimodelCreateTopicRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create Async Messaging Topic (CreateTopicHandler)

    Create New Async Messaging Topic, the topic name is unique inside a game namespace.

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/asyncmessaging/topics

        method: POST

        tags: ["Async Messaging"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelCreateTopicRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelTopicResponse (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        409: Conflict - ResponseErrorResponse (Conflict)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = CreateTopicHandler.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteTopicHandler)
def delete_topic_handler(
    topic_name: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Async Messaging Topic (DeleteTopicHandler)

    Delete Async Messaging Topic inside a game namespace given its topic name.

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/asyncmessaging/topics/{topicName}

        method: DELETE

        tags: ["Async Messaging"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        topic_name: (topicName) REQUIRED str in path

    Responses:
        200: OK - ApimodelTopicResponse (OK)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = DeleteTopicHandler.create(
        topic_name=topic_name,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteTopicHandler)
async def delete_topic_handler_async(
    topic_name: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Async Messaging Topic (DeleteTopicHandler)

    Delete Async Messaging Topic inside a game namespace given its topic name.

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/asyncmessaging/topics/{topicName}

        method: DELETE

        tags: ["Async Messaging"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        topic_name: (topicName) REQUIRED str in path

    Responses:
        200: OK - ApimodelTopicResponse (OK)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = DeleteTopicHandler.create(
        topic_name=topic_name,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(ListTopicsHandler)
def list_topics_handler(
    fuzzy_topic_name: Optional[str] = None,
    is_subscribed_by_app_name: Optional[str] = None,
    is_unsubscribed_by_app_name: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List Async Messaging Topics (ListTopicsHandler)

    List all Async Messaging Topics inside a game namespace.

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/asyncmessaging/topics

        method: GET

        tags: ["Async Messaging"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        fuzzy_topic_name: (fuzzyTopicName) OPTIONAL str in query

        is_subscribed_by_app_name: (isSubscribedByAppName) OPTIONAL str in query

        is_unsubscribed_by_app_name: (isUnsubscribedByAppName) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - ApimodelListTopicsResponse (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = ListTopicsHandler.create(
        fuzzy_topic_name=fuzzy_topic_name,
        is_subscribed_by_app_name=is_subscribed_by_app_name,
        is_unsubscribed_by_app_name=is_unsubscribed_by_app_name,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ListTopicsHandler)
async def list_topics_handler_async(
    fuzzy_topic_name: Optional[str] = None,
    is_subscribed_by_app_name: Optional[str] = None,
    is_unsubscribed_by_app_name: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List Async Messaging Topics (ListTopicsHandler)

    List all Async Messaging Topics inside a game namespace.

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/asyncmessaging/topics

        method: GET

        tags: ["Async Messaging"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        fuzzy_topic_name: (fuzzyTopicName) OPTIONAL str in query

        is_subscribed_by_app_name: (isSubscribedByAppName) OPTIONAL str in query

        is_unsubscribed_by_app_name: (isUnsubscribedByAppName) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - ApimodelListTopicsResponse (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = ListTopicsHandler.create(
        fuzzy_topic_name=fuzzy_topic_name,
        is_subscribed_by_app_name=is_subscribed_by_app_name,
        is_unsubscribed_by_app_name=is_unsubscribed_by_app_name,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UnsubscribeTopicHandler)
def unsubscribe_topic_handler(
    app: str,
    topic_name: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Unsubscribe From Topic (UnsubscribeTopicHandler)

    Unsubscribe App queue from a topic by Topic Name.

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/asyncmessaging/topics/{topicName}/subscriptions

        method: DELETE

        tags: ["Async Messaging"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        topic_name: (topicName) REQUIRED str in path

    Responses:
        200: OK - ApimodelTopicSubscription (OK)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = UnsubscribeTopicHandler.create(
        app=app,
        topic_name=topic_name,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UnsubscribeTopicHandler)
async def unsubscribe_topic_handler_async(
    app: str,
    topic_name: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Unsubscribe From Topic (UnsubscribeTopicHandler)

    Unsubscribe App queue from a topic by Topic Name.

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/asyncmessaging/topics/{topicName}/subscriptions

        method: DELETE

        tags: ["Async Messaging"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        topic_name: (topicName) REQUIRED str in path

    Responses:
        200: OK - ApimodelTopicSubscription (OK)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = UnsubscribeTopicHandler.create(
        app=app,
        topic_name=topic_name,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Session History Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ModelsMatchingRule(Model):
    """Models matching rule (models.MatchingRule)

    Properties:
        attribute: (attribute) REQUIRED str

        criteria: (criteria) REQUIRED str

        reference: (reference) REQUIRED float
    """

    # region fields

    attribute: str  # REQUIRED
    criteria: str  # REQUIRED
    reference: float  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_attribute(self, value: str) -> ModelsMatchingRule:
        self.attribute = value
        return self

    def with_criteria(self, value: str) -> ModelsMatchingRule:
        self.criteria = value
        return self

    def with_reference(self, value: float) -> ModelsMatchingRule:
        self.reference = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "attribute"):
            result["attribute"] = str(self.attribute)
        elif include_empty:
            result["attribute"] = ""
        if hasattr(self, "criteria"):
            result["criteria"] = str(self.criteria)
        elif include_empty:
            result["criteria"] = ""
        if hasattr(self, "reference"):
            result["reference"] = float(self.reference)
        elif include_empty:
            result["reference"] = 0.0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls, attribute: str, criteria: str, reference: float, **kwargs
    ) -> ModelsMatchingRule:
        instance = cls()
        instance.attribute = attribute
        instance.criteria = criteria
        instance.reference = reference
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsMatchingRule:
        instance = cls()
        if not dict_:
            return instance
        if "attribute" in dict_ and dict_["attribute"] is not None:
            instance.attribute = str(dict_["attribute"])
        elif include_empty:
            instance.attribute = ""
        if "criteria" in dict_ and dict_["criteria"] is not None:
            instance.criteria = str(dict_["criteria"])
        elif include_empty:
            instance.criteria = ""
        if "reference" in dict_ and dict_["reference"] is not None:
            instance.reference = float(dict_["reference"])
        elif include_empty:
            instance.reference = 0.0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsMatchingRule]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsMatchingRule]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsMatchingRule, List[ModelsMatchingRule], Dict[Any, ModelsMatchingRule]
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "attribute": "attribute",
            "criteria": "criteria",
            "reference": "reference",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "attribute": True,
            "criteria": True,
            "reference": True,
        }

    # endregion static methods
